﻿local addonName, addonTable = ...;
--[[ Here you can define the slash commands by inserting them into a table.

    The table consists of 1 child tables and some data attributes:
        desc        with the general description/headline for this slash command
        slashcmd    the name of index of the SlashCmdList[index]
        printall    switch for printing hidden commands (true or nil)
        cmds        a table with all commands your addon can process
    The cmds subtable must contain for each command following attributes:
        cmd         contains the command name as you use it, only one word is optimal to process
    The cmds subtable can contain further these attributes:
        param       contains the possible params
        desc        contains the description for the command
        show        true if you want to print it as help per default
        indent      a number, how often your desc should be indented (it ommits /slashcommand then)
        func        contains the called function for this command

    Please be aware, that we use always the function of the first inserted command, if there are
    more than one with the same cmd attribute.

    -- Example for: /ase command param1 param2 param3
    SlashCmdList["ASE"] = function(msg)
        // do whatever you wish with the chat commands in msg
        // local command, params = msg:match("^(%S*)%s*(.-)$"); -- Divide into command and parameter
    end
    SLASH_ASE1 = "/ase";
    SLASH_ASE2 = "/aslashexample";
    local yourCmdTable = {
        desc = "Your Slash command Help Header----------",
        cmds = { },
    };
    tinsert(yourCmdTable.cmds, {
        cmd = "command1",
        desc = "Just an example command",
        show = true,
        func = function(args)
            print("Hello world -> " .. args);
        end }
    );

--]]

-- General functions
local COLOR_GREYEDOUT = "808080";
local COLOR_INDENT    = "C0C0C0";

local function getFunc( cmdTable, cmd )
    local func;
    if cmd == nil or cmd == "" then
        return func;
    end
    for i=1, table.getn(cmdTable.cmds) do
        if ( cmdTable.cmds[i].cmd == cmd ) then
            func = cmdTable.cmds[i].func;
            break;
        end
    end
    return func;
end

local function printHelp( cmdTable )
    local cmdlist = "SLASH_" .. cmdTable.slashcmd;
    local txt = "";

    local index = 1;
    while getglobal(cmdlist .. index) ~= nil do
        txt = txt .. "  " .. getglobal(cmdlist .. index);

        index = index + 1;
    end
    print(cmdTable.desc);
    print("Slash commands: " .. txt);

    local main = getglobal(cmdlist .. "1");
    for cmd, t in pairs(cmdTable.cmds) do
        if t.show or cmdTable.printall then
            local help = "";
            if t.cmd ~= nil and t.cmd ~= "" then
                local desc = "";
                local param = t.param or "";
                if param ~= "" then
                    param = " " .. param;
                end
                if t.desc then
                    -- desc = desc .. " --> " .. t.desc;
                    desc = t.desc;
                end
                if t.indent and tonumber(t.indent) and t.indent > 0 then
                    help = "|cFF"..COLOR_INDENT .. help;
                    -- help = help .. strrep("    ",t.indent) .. t.cmd .. param .. desc;
                    help = help .. desc;
                    help = help .. "|r";
                else
                    help = main .. " " .. t.cmd .. param .. "\n   " .. desc;
                    -- help = desc;
                end
                if t.show == nil or t.show == false then
                    help = "|cFF" .. COLOR_GREYEDOUT .. help .. "|r";
                end
            else
                help = "|cFF90FF90~"..t.desc .. "~|r";
            end

            print(help);
        end
    end
end



-- Slash handling for /niill, cmdTableNI

local cmdTableNI = {};
SlashCmdList["NIILL"] = function( msg, ... ) -- cmdTable, args
    local command, args = msg:match("^(%S*)%s*(.-)$");

	local func = getFunc(cmdTableNI, command);
	
	if ( func ~= nil ) then
		func(args);
    else
        printHelp( cmdTableNI );
    end
end
SLASH_NIILL1 = "/niill";
SLASH_NIILL2 = "/nightillusion";

cmdTableNI = {
    desc = strsub(NightIllusion_L["CMD_HELP"],1,42);
    slashcmd = "NIILL",
    cmds = { },
    -- printall = true;
};
tinsert( cmdTableNI.cmds, {
    desc = "RP Flag",
    -- show = true,
    }
);
tinsert( cmdTableNI.cmds, { 
    cmd = "minalpha",
    param = "value",
    desc = NightIllusion_L["CMD_HELP_MINALPHA"],
    show = true,
    func = function(args) 
        local alpha = args;
        if tonumber(alpha) then
            alpha = tonumber(alpha);
            if alpha >= 0 and alpha <= 1 then
                NIDisplay:SetMinAlpha( alpha );
                -- NICore:PrintM( "Alpha set to " .. alpha , "" );
                NICore:PrintD("Minimal Alpha is " .. alpha, "DEB");
                return;
            end
        end
        NICore:PrintE(NightIllusion_L["ERR_ALPHA"], "ERR");
    end
    }
);
tinsert( cmdTableNI.cmds, {
    cmd = "maxalpha",
    param = "value",
    desc = NightIllusion_L["CMD_HELP_MAXALPHA"],
    show = true,
    func = function(args) 
        local alpha = args;
        
        if tonumber(alpha) then
            alpha = tonumber(alpha);
            if alpha >= 0 and alpha <= 1 then
                NIDisplay:SetMaxAlpha( alpha );
                NICore:PrintD("Maximal Alpha is " .. alpha, "DEB");
                return;
            end
        end
        
        NICore:PrintE(NightIllusion_L["ERR_ALPHA"], "ERR");
    end
    }
);
tinsert( cmdTableNI.cmds, { 
    cmd = "startnight",
    param = "00:00 [a.m./p.m.]",
    desc = NightIllusion_L["CMD_HELP_STARTNIGHT"],
    show = true,
    func = function(args)
        local hh, mm = NIDisplay:Calc24H( args );

        if hh >= 0 then
            NIDisplay:SetStartTime(hh, mm);
            NICore:PrintD("Starting Night at " .. hh ..":"..mm, "DEB");
        else
            NICore:PrintE(NightIllusion_L["ERR_TIME"], "ERR");
        end
        
    end
    }
);
tinsert( cmdTableNI.cmds, {
    cmd = "endnight", 
    param = "00:00 [a.m./p.m.]",
    desc = NightIllusion_L["CMD_HELP_ENDNIGHT"],
    show = true,
    func = function(args)
        local hh, mm = NIDisplay:Calc24H( args );

        if hh >= 0 then
            NIDisplay:SetEndTime(hh, mm);
            NICore:PrintD("Ending Night at " .. hh ..":"..mm, "DEB");
        else
            NICore:PrintE(NightIllusion_L["ERR_TIME"], "ERR");
        end
        
    end
    }
);
tinsert( cmdTableNI.cmds, {
    cmd = "off", 
    desc = NightIllusion_L["CMD_HELP_OFF"],
    show = true,
    func = function(args)
        NIDisplay:SetActive(false);
        NICore:PrintD("NightIllusion disabled", "DEB");
    end
    }
);
tinsert( cmdTableNI.cmds, {
    cmd = "on", 
    desc = NightIllusion_L["CMD_HELP_ON"],
    show = true,
    func = function(args)
        NIDisplay:SetActive(true);
        NICore:PrintD("NightIllusion enabled", "DEB");
    end
    }
);
tinsert( cmdTableNI.cmds, {
    cmd = "state", 
    desc = NightIllusion_L["CMD_HELP_ON"],
    func = function(args)
        NIDisplay:PrintState();
    end
    }
);
tinsert( cmdTableNI.cmds, {
    cmd = "shadow", 
    param = "transparency",
    desc = NightIllusion_L["CMD_HELP_ALPHA"],
    show = true,
    func = function(args)
        if tonumber(args) and tonumber(args) >= 0.0 and tonumber(args) <= 1.0 then
            NIDisplay:SetAlpha(tonumber(args));
            NICore:PrintD("Set shadow transparency to " .. tonumber(args), "DEB");
        else
            NICore:PrintE(NightIllusion_L["ERR_ALPHA"], "ERR");
        end
    end
    }
);
tinsert( cmdTableNI.cmds, {
    cmd = "resetshadow", 
    desc = NightIllusion_L["CMD_HELP_RESETAPHA"],
    show = true,
    func = function(args)
        NIDisplay:SetAlpha( NIDisplay:CalculateAlphaShadow() );
        NICore:PrintD("Reset shadow transparency", "DEB");
    end
    }
);